from netgen.csg import *
from ngsolve import *

geo = CSGeometry()
beam = OrthoBrick(Pnt(-1,0,0),Pnt(11,1,1))
beam *= Plane(Pnt(0,0,0),Vec(-1,0,0)).bc("wall")
beam *= Plane(Pnt(10,0,0),Vec(1,0,0)).bc("force")
geo.Add(beam)
mesh = Mesh(geo.GenerateMesh(maxh=0.2))

fes = H1(mesh, order=2, dirichlet="wall", dim=mesh.dim)

u,v = fes.TrialFunction(), fes.TestFunction()

# linear strain tensor
def epsilon(u):
     return 0.5 * (u.Deriv().trans + u.Deriv())

# material parameters
E, nu = 2.1e11, 0.2
# lame parameters
mu  = 1/2*(E / (1+nu))
lam = E * nu / ((1+nu)*(1-2*nu))

a = BilinearForm(fes)
a += SymbolicBFI( 2*mu*InnerProduct(epsilon(u),epsilon(v)) + lam*Trace(u.Deriv())*Trace(v.Deriv()))
a.Assemble()

force = CoefficientFunction( (0,1e5,0) )

f = LinearForm(fes)
f += SymbolicLFI( force*v, definedon=mesh.Boundaries("force"))
f.Assemble()

u = GridFunction(fes,name="displacement")
u.vec.data = a.mat.Inverse(fes.FreeDofs())*f.vec
Draw(u)

# linear stress tensor
def stress(u):
     return 2*mu*epsilon(u) + lam*Trace(epsilon(u))*Id(mesh.dim)
Draw(stress(u),mesh,"stress")
